import { types as t } from "@babel/core";
import { declare } from "@babel/helper-plugin-utils";
import template from "@babel/template";
const DefineAccessorHelper = template.expression.ast`
function (type, obj, key, fn) {
  var desc = { configurable: true, enumerable: true };
  desc[type] = fn;
  return Object.defineProperty(obj, key, desc);
}`;
DefineAccessorHelper._compact = true;
export default declare((api, options) => {
  var _api$assumption;
  api.assertVersion(7);
  const setComputedProperties = (_api$assumption = api.assumption("setComputedProperties")) != null ? _api$assumption : options.loose;
  const pushComputedProps = setComputedProperties ? pushComputedPropsLoose : pushComputedPropsSpec;
  function buildDefineAccessor(state, type, obj, key, fn) {
    let helper;
    if (state.availableHelper("defineAccessor")) {
      helper = state.addHelper("defineAccessor");
    } else {
      const file = state.file;
      helper = file.get("fallbackDefineAccessorHelper");
      if (!helper) {
        const id = file.scope.generateUidIdentifier("defineAccessor");
        file.scope.push({
          id,
          init: DefineAccessorHelper
        });
        file.set("fallbackDefineAccessorHelper", helper = id);
      }
      helper = t.cloneNode(helper);
    }
    return t.callExpression(helper, [t.stringLiteral(type), obj, key, fn]);
  }
  function getValue(prop) {
    if (t.isObjectProperty(prop)) {
      return prop.value;
    } else if (t.isObjectMethod(prop)) {
      return t.functionExpression(null, prop.params, prop.body, prop.generator, prop.async);
    }
  }
  function pushAssign(objId, prop, body) {
    body.push(t.expressionStatement(t.assignmentExpression("=", t.memberExpression(t.cloneNode(objId), prop.key, prop.computed || t.isLiteral(prop.key)), getValue(prop))));
  }
  function pushAccessorDefine({
    body,
    computedProps,
    initPropExpression,
    objId,
    state
  }, prop) {
    const kind = prop.kind;
    const key = !prop.computed && t.isIdentifier(prop.key) ? t.stringLiteral(prop.key.name) : prop.key;
    const value = getValue(prop);
    if (computedProps.length === 1) {
      return buildDefineAccessor(state, kind, initPropExpression, key, value);
    } else {
      body.push(t.expressionStatement(buildDefineAccessor(state, kind, t.cloneNode(objId), key, value)));
    }
  }
  function pushComputedPropsLoose(info) {
    for (const prop of info.computedProps) {
      if (t.isObjectMethod(prop) && (prop.kind === "get" || prop.kind === "set")) {
        const single = pushAccessorDefine(info, prop);
        if (single) return single;
      } else {
        pushAssign(t.cloneNode(info.objId), prop, info.body);
      }
    }
  }
  function pushComputedPropsSpec(info) {
    const {
      objId,
      body,
      computedProps,
      state
    } = info;
    for (const prop of computedProps) {
      const key = t.toComputedKey(prop);
      if (t.isObjectMethod(prop) && (prop.kind === "get" || prop.kind === "set")) {
        const single = pushAccessorDefine(info, prop);
        if (single) return single;
      } else {
        const value = getValue(prop);
        if (computedProps.length === 1) {
          return t.callExpression(state.addHelper("defineProperty"), [info.initPropExpression, key, value]);
        } else {
          body.push(t.expressionStatement(t.callExpression(state.addHelper("defineProperty"), [t.cloneNode(objId), key, value])));
        }
      }
    }
  }
  return {
    name: "transform-computed-properties",
    visitor: {
      ObjectExpression: {
        exit(path, state) {
          const {
            node,
            parent,
            scope
          } = path;
          let hasComputed = false;
          for (const prop of node.properties) {
            hasComputed = prop.computed === true;
            if (hasComputed) break;
          }
          if (!hasComputed) return;
          const initProps = [];
          const computedProps = [];
          let foundComputed = false;
          for (const prop of node.properties) {
            if (t.isSpreadElement(prop)) {
              continue;
            }
            if (prop.computed) {
              foundComputed = true;
            }
            if (foundComputed) {
              computedProps.push(prop);
            } else {
              initProps.push(prop);
            }
          }
          const objId = scope.generateUidIdentifierBasedOnNode(parent);
          const initPropExpression = t.objectExpression(initProps);
          const body = [];
          body.push(t.variableDeclaration("var", [t.variableDeclarator(objId, initPropExpression)]));
          const single = pushComputedProps({
            scope,
            objId,
            body,
            computedProps,
            initPropExpression,
            state
          });
          if (single) {
            path.replaceWith(single);
          } else {
            body.push(t.expressionStatement(t.cloneNode(objId)));
            path.replaceWithMultiple(body);
          }
        }
      }
    }
  };
});

//# sourceMappingURL=index.js.map
